@@ -66,6 +66,7 @@ typedef enum {
    GHI_CAP_TRAY_ICONS                   = 19, // supports ghi.guest.trayIcon commands
    GHI_CAP_SET_FOCUSED_WINDOW           = 20, // supports ghi.guest.setFocusedWindow
    GHI_CAP_GET_EXEC_INFO_HASH           = 21, // supports ghi.guest.getExecInfoHash
+   UNITY_CAP_STICKY_WINDOWS             = 22, // supports unity.window.{un,}stick
 } GuestCapabilities;
 
 typedef struct {
@@ -129,6 +130,7 @@ static GuestCapElem guestCapTable[] = {
    { GHI_CAP_TRAY_ICONS,                   GHI_CAP_VMDB_PATH,   "trayIcons" },
    { GHI_CAP_SET_FOCUSED_WINDOW,           GHI_CAP_VMDB_PATH,   "setFocusedWindow"},
    { GHI_CAP_GET_EXEC_INFO_HASH,           GHI_CAP_VMDB_PATH,   "getExecInfoHash"},
+   { UNITY_CAP_STICKY_WINDOWS,             UNITY_CAP_VMDB_PATH, "sticky"},
 };
 
 #endif // VM_NEED_VMDB_GUEST_CAP_MAPPING
