@@ -29,8 +29,6 @@
 
 typedef void (LogBasicFunc)(const char *fmt, va_list args);
 
-struct LogState;
-
 typedef struct
 {
    const char *fileName;             // File name, if known
@@ -50,7 +48,8 @@ typedef struct
 EXTERN void Log_GetInitDefaults(const char *fileName, const char *config,
                                 const char *suffix, LogInitParams *params);
 
-EXTERN Bool Log_Init(const char *fileName, const char *config, const char *suffix);
+EXTERN Bool Log_Init(const char *fileName, const char *config,
+                     const char *suffix);
 EXTERN Bool Log_InitForApp(const char *fileName, const char *config,
                            const char *suffix, const char *appName,
                            const char *appVersion);
@@ -108,10 +107,12 @@ EXTERN void GuestLog_Log(const char *fmt, ...) PRINTF_DECL(1, 2);
 #define LOG_DEFAULT_KEEPOLD 3
 #endif
 
+#define LOG_NO_BPS_LIMIT        0xFFFFFFFF
+
 #ifdef VMX86_RELEASE
 #define LOG_DEFAULT_THROTTLE_BPS       1000
 #else
-#define LOG_DEFAULT_THROTTLE_BPS       0
+#define LOG_DEFAULT_THROTTLE_BPS       LOG_NO_BPS_LIMIT
 #endif
 
 #define LOG_DEFAULT_THROTTLE_THRESHOLD 1000000
