@@ -893,6 +893,13 @@ typedef uint32 ToolsVersion;
 #define   TOOLS_VERSION_WS70_BETA2_V_MNR  1
 #define   TOOLS_VERSION_WS70_BETA2_V_BASE 1
 
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_WS70_RC  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_WS70_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_WS70_RC_V_MJR  8
+#define   TOOLS_VERSION_WS70_RC_V_MNR  1
+#define   TOOLS_VERSION_WS70_RC_V_BASE 2
+
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_FUSION30_BETA1  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_FUSION30_BETA1_V)
 #endif /* RC_INVOKED */
@@ -907,10 +914,16 @@ typedef uint32 ToolsVersion;
 #define   TOOLS_VERSION_FUSION30_BETA2_V_MNR  2
 #define   TOOLS_VERSION_FUSION30_BETA2_V_BASE 1
 
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_FUSION30_RC  TOOLS_VERSION_TO_UINT(TOOLS_VERSION_FUSION30_RC_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_FUSION30_RC_V_MJR  8
+#define   TOOLS_VERSION_FUSION30_RC_V_MNR  2
+#define   TOOLS_VERSION_FUSION30_RC_V_BASE 2
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_WS70_BETA2
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_WS70_BETA2)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_WS70_BETA2)
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_WS70_RC
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_WS70_RC)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_WS70_RC)
 
 
 /*
