@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2009-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2009 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -27,7 +27,6 @@
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
-#include <net/if.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
@@ -218,8 +217,8 @@ SlashProcNet_GetSnmp(void)
 {
    GHashTable *myHashTable = NULL;
    GIOChannel *myChannel = NULL;
-   GIOStatus keyIoStatus = G_IO_STATUS_ERROR;
-   GIOStatus valIoStatus = G_IO_STATUS_ERROR;
+   GIOStatus keyIoStatus;
+   GIOStatus valIoStatus;
    gchar *myKeyLine = NULL;
    gchar *myValLine = NULL;
    Bool parseError = FALSE;
@@ -374,7 +373,8 @@ badIteration:
    myValLine = NULL;
 
    close(fd);
-   g_io_channel_unref(myChannel);
+
+   g_free(myChannel);
 
    return myHashTable;
 }
@@ -485,7 +485,8 @@ SlashProcNet_GetSnmp6(void)
    }
 
    close(fd);
-   g_io_channel_unref(myChannel);
+
+   g_free(myChannel);
 
    return myHashTable;
 }
@@ -537,7 +538,7 @@ SlashProcNet_GetRoute(void)
       myFieldsRE = g_regex_new("^Iface\\s+Destination\\s+Gateway\\s+Flags\\s+"
                                "RefCnt\\s+Use\\s+Metric\\s+Mask\\s+MTU\\s+"
                                "Window\\s+IRTT\\s*$", 0, 0, NULL);
-      myValuesRE = g_regex_new("^(\\S+)\\s+([[:xdigit:]]{8})\\s+"
+      myValuesRE = g_regex_new("^(\\w+)\\s+([[:xdigit:]]{8})\\s+"
                                "([[:xdigit:]]{8})\\s+([[:xdigit:]]{4})\\s+"
                                "\\d+\\s+\\d+\\s+(\\d+)\\s+"
                                "([[:xdigit:]]{8})\\s+(\\d+)\\s+\\d+\\s+(\\d+)\\s*$",
@@ -642,8 +643,9 @@ badIteration:
 
 out:
    g_free(myLine);
+   g_free(myChannel);
+
    close(fd);
-   g_io_channel_unref(myChannel);
 
    return myArray;
 }
@@ -728,7 +730,7 @@ SlashProcNet_GetRoute6(void)
                                 "([[:xdigit:]]{32}) ([[:xdigit:]]{2}) "
                                 "([[:xdigit:]]{32}) ([[:xdigit:]]{8}) "
                                 "[[:xdigit:]]{8} [[:xdigit:]]{8} "
-                                "([[:xdigit:]]{8})\\s+(\\S+)\\s*$", 0, 0,
+                                "([[:xdigit:]]{8})\\s+(\\w+)\\s*$", 0, 0,
                                 NULL);
       ASSERT(myValuesRE);
    }
@@ -769,7 +771,7 @@ SlashProcNet_GetRoute6(void)
       myEntry->rtmsg_metric = MatchToGuint64(myMatchInfo, 6, 16);
       myEntry->rtmsg_flags = MatchToGuint64(myMatchInfo, 7, 16);
 
-      MATCHEXPR(myMatchInfo, 8, myEntry->rtmsg_ifindex = if_nametoindex(MATCH));
+      MATCHEXPR(myMatchInfo, 8, myEntry->rtmsg_ifindex = NetUtil_GetIfIndex(MATCH));
 
 badIteration:
       g_free(myLine);
@@ -784,15 +786,14 @@ badIteration:
    }
 
    if (myArray && myIoStatus != G_IO_STATUS_EOF) {
-      SlashProcNet_FreeRoute6(myArray);
+      g_ptr_array_free(myArray, TRUE);
       myArray = NULL;
    }
 
    g_free(myLine);
    myLine = NULL;
-
-   close(fd);
-   g_io_channel_unref(myChannel);
+   g_free(myChannel);
+   myChannel = NULL;
 
    return myArray;
 }
