@@ -52,7 +52,8 @@ static struct inode *HgfsInodeLookup(struct super_block *sb,
 static void HgfsSetFileType(struct inode *inode,
                             HgfsAttrInfo const *attr);
 static int HgfsUnpackGetattrReply(HgfsReq *req,
-                                  HgfsAttrInfo *attr);
+                                  HgfsAttrInfo *attr,
+                                  char **fileName);
 static int HgfsPackGetattrRequest(HgfsReq *req,
                                   struct dentry *dentry,
                                   Bool allowHandleReuse,
@@ -227,7 +228,8 @@ HgfsSetFileType(struct inode *inode,          // IN/OUT: Inode to update
  */
 static int
 HgfsUnpackGetattrReply(HgfsReq *req,        // IN: Reply packet
-                       HgfsAttrInfo *attr)  // IN/OUT: Attributes
+                       HgfsAttrInfo *attr,  // IN/OUT: Attributes
+                       char **fileName)     // OUT: file name
 {
    int result;
    char *name = NULL;
@@ -267,19 +269,23 @@ HgfsUnpackGetattrReply(HgfsReq *req,        // IN: Reply packet
       }
    }
 
-   if (length != 0) {
+   if (fileName) {
+      if (length != 0) {
 
-      attr->fileName = kmalloc(length + 1, GFP_KERNEL);
-      if (attr->fileName == NULL) {
-         LOG(4, (KERN_DEBUG "VMware hgfs: HgfsUnpackGetattrReply: out of "
-                 "memory allocating symlink target name, ignoring\n"));
-         return -ENOMEM;
+         *fileName = kmalloc(length + 1, GFP_KERNEL);
+         if (*fileName == NULL) {
+            LOG(4, (KERN_DEBUG "VMware hgfs: HgfsUnpackGetattrReply: out of "
+                    "memory allocating symlink target name, ignoring\n"));
+            return -ENOMEM;
+         }
+
+         /* Copy and convert. From now on, the symlink target is in UTF8. */
+         memcpy(*fileName, name, length);
+         CPNameLite_ConvertFrom(*fileName, length, '/');
+         (*fileName)[length] = '\0';
+      } else {
+         *fileName = NULL;
       }
-
-      /* Copy and convert. From now on, the symlink target is in UTF8. */
-      memcpy(attr->fileName, name, length);
-      CPNameLite_ConvertFrom(attr->fileName, length, '/');
-      attr->fileName[length] = '\0';
    }
 
    return 0;
@@ -570,6 +576,10 @@ HgfsUnpackCommonAttr(HgfsReq *req,            // IN: Reply packet
          attrInfo->hostFileId = attrV2->hostFileId;
          attrInfo->mask |= HGFS_ATTR_VALID_FILEID;
       }
+      if (attrV2->mask & HGFS_ATTR_VALID_EFFECTIVE_PERMS) {
+         attrInfo->effectivePerms = attrV2->effectivePerms;
+         attrInfo->mask |= HGFS_ATTR_VALID_EFFECTIVE_PERMS;
+      }
    } else if (attrV1 != NULL) {
       /* Implicit mask for a Version 1 attr. */
       attrInfo->mask = HGFS_ATTR_VALID_TYPE |
@@ -577,7 +587,8 @@ HgfsUnpackCommonAttr(HgfsReq *req,            // IN: Reply packet
          HGFS_ATTR_VALID_ACCESS_TIME |
          HGFS_ATTR_VALID_WRITE_TIME |
          HGFS_ATTR_VALID_CHANGE_TIME |
-         HGFS_ATTR_VALID_OWNER_PERMS;
+         HGFS_ATTR_VALID_OWNER_PERMS |
+         HGFS_ATTR_VALID_EFFECTIVE_PERMS;
 
       attrInfo->type = attrV1->type;
       attrInfo->size = attrV1->size;
@@ -585,6 +596,7 @@ HgfsUnpackCommonAttr(HgfsReq *req,            // IN: Reply packet
       attrInfo->writeTime = attrV1->writeTime;
       attrInfo->attrChangeTime = attrV1->attrChangeTime;
       attrInfo->ownerPerms = attrV1->permissions;
+      attrInfo->effectivePerms = attrV1->permissions;
    }
 
    return 0;
@@ -699,7 +711,7 @@ HgfsChangeFileAttributes(struct inode *inode,          // IN/OUT: Inode
    }
 
    inode->i_rdev = 0;  /* Device nodes are not supported */
-#if !defined(VMW_INODE_2618)
+#if !defined VMW_INODE_2618
    inode->i_blksize = HGFS_BLOCKSIZE;
 #endif
 
@@ -784,8 +796,8 @@ HgfsChangeFileAttributes(struct inode *inode,          // IN/OUT: Inode
  *    for the indicated remote name, and if it succeeds copy the
  *    results of the getattr into the provided HgfsAttrInfo.
  *
- *    attr->fileName will be allocated on success if the file is a
- *    symlink; it's the caller's duty to free it.
+ *    fileName (if supplied) will be set to a newly allocated string
+ *    if the file is a symlink; it's the caller's duty to free it.
  *
  * Results:
  *    Returns zero on success, or a negative error on failure.
@@ -798,7 +810,8 @@ HgfsChangeFileAttributes(struct inode *inode,          // IN/OUT: Inode
 
 int
 HgfsPrivateGetattr(struct dentry *dentry,  // IN: Dentry containing name
-                   HgfsAttrInfo *attr)     // OUT: Attr to copy into
+                   HgfsAttrInfo *attr,     // OUT: Attr to copy into
+                   char **fileName)        // OUT: pointer to allocated file name
 {
    struct HgfsSuperInfo *si;
    HgfsReq *req;
@@ -825,7 +838,7 @@ HgfsPrivateGetattr(struct dentry *dentry,  // IN: Dentry containing name
 
   retry:
 
-   opUsed = atomic_read(&hgfsVersionGetattr);
+   opUsed = hgfsVersionGetattr;
    result = HgfsPackGetattrRequest(req, dentry, allowHandleReuse, opUsed, attr);
    if (result != 0) {
       LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPrivateGetattr: no attrs\n"));
@@ -844,7 +857,7 @@ HgfsPrivateGetattr(struct dentry *dentry,  // IN: Dentry containing name
        */
       switch (result) {
       case 0:
-         result = HgfsUnpackGetattrReply(req, attr);
+         result = HgfsUnpackGetattrReply(req, attr, fileName);
          break;
       case -EBADF:
          /*
@@ -872,12 +885,12 @@ HgfsPrivateGetattr(struct dentry *dentry,  // IN: Dentry containing name
          if (attr->requestType == HGFS_OP_GETATTR_V3) {
             LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPrivateGetattr: Version 3 "
                     "not supported. Falling back to version 2.\n"));
-            atomic_set(&hgfsVersionGetattr, HGFS_OP_GETATTR_V2);
+            hgfsVersionGetattr = HGFS_OP_GETATTR_V2;
             goto retry;
          } else if (attr->requestType == HGFS_OP_GETATTR_V2) {
             LOG(4, (KERN_DEBUG "VMware hgfs: HgfsPrivateGetattr: Version 2 "
                     "not supported. Falling back to version 1.\n"));
-            atomic_set(&hgfsVersionGetattr, HGFS_OP_GETATTR);
+            hgfsVersionGetattr = HGFS_OP_GETATTR;
             goto retry;
          }
 
@@ -1020,6 +1033,9 @@ HgfsIget(struct super_block *sb,         // IN: Superblock of this fs
       }
 
       iinfo = INODE_GET_II_P(inode);
+      if (attr->mask & HGFS_ATTR_VALID_FILEID) {
+         iinfo->hostFileId = attr->hostFileId;
+      }
       iinfo->isFakeInodeNumber = isFakeInodeNumber;
       iinfo->isReferencedInode = TRUE;
       HgfsChangeFileAttributes(inode, attr);
@@ -1067,12 +1083,10 @@ HgfsInstantiate(struct dentry *dentry,    // IN: Dentry to use
       int error;
 
       LOG(6, (KERN_DEBUG "VMware hgfs: HgfsInstantiate: issuing getattr\n"));
-      newAttr.fileName = NULL;
-      error = HgfsPrivateGetattr(dentry, &newAttr);
+      error = HgfsPrivateGetattr(dentry, &newAttr, NULL);
       if (error) {
          return error;
       }
-      kfree(newAttr.fileName);
       attr = &newAttr;
    }
 
@@ -1751,6 +1765,7 @@ HgfsDoReadInode(struct inode *inode)  // IN: Inode to initialize
 #endif
    INODE_SET_II_P(inode, iinfo);
    INIT_LIST_HEAD(&iinfo->files);
+   iinfo->hostFileId = 0;
    iinfo->isReferencedInode = FALSE;
    iinfo->isFakeInodeNumber = FALSE;
    iinfo->createdAndUnopened = FALSE;
