@@ -112,6 +112,10 @@ ToolsCore_InitRpc(ToolsServiceState *state);
 gboolean
 ToolsCore_LoadPlugins(ToolsServiceState *state);
 
+void
+ToolsCore_ReloadConfig(ToolsServiceState *state,
+                       gboolean force);
+
 void
 ToolsCore_RegisterPlugins(ToolsServiceState *state);
 
