@@ -1,5 +1,5 @@
 /*********************************************************
- * Copyright (C) 2006-2015 VMware, Inc. All rights reserved.
+ * Copyright (C) 2006 VMware, Inc. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -48,11 +48,12 @@
 #include "vmblock_user.h"
 #include "vm_basic_types.h"
 
+#define BLOCKROOT                 VMBLOCK_MOUNT_POINT "/"
 #define CONTROLFILE               VMBLOCK_DEVICE
 #define CONTROLFILE_MODE          VMBLOCK_DEVICE_MODE
 #define REALROOT                   "/tmp/VMwareDnD/"
 #define FILENAME                   "/foo"
-#define ACCESSORFULLNAME(dir)      VMBLOCK_FS_ROOT "/" dir FILENAME
+#define ACCESSORFULLNAME(dir)      BLOCKROOT dir FILENAME
 #define BLOCKERFULLNAME(dir)       REALROOT dir
 
 #define lprintf(...)                                                    \
