@@ -445,10 +445,9 @@ string::operator const ubstr_t()
  */
 
 string&
-string::operator=(string copy) // IN
+string::operator=(const string &s) // IN
 {
-   swap(copy);
-   return *this;
+   return assign(s);
 }
 
 
@@ -1014,8 +1013,6 @@ string::push_back(value_type uc) // IN
  *
  *      Assigns the passed in string to this string.
  *
- *      Callers should prefer using operator= instead of assign().
- *
  * Results:
  *      A reference to this object
  *
@@ -1028,7 +1025,9 @@ string::push_back(value_type uc) // IN
 string&
 string::assign(const string &s) // IN
 {
-   return operator=(s);
+   string copy(s);
+   swap(copy);
+   return *this;
 }
 
 
@@ -1227,11 +1226,10 @@ string::replace(size_type i,     // IN
  *
  * utf::string::replace --
  *
- *      Mutates this string by replacing all occurrences of one string with
- *      another.
+ *      Replace all occurences of the given string with another string.
  *
  * Results:
- *      A reference to this object.
+ *      A utf::string created with the substring replaced.
  *
  * Side effects:
  *      None
@@ -1264,30 +1262,6 @@ string::replace(const string &from, // IN
 }
 
 
-/*
- *-----------------------------------------------------------------------------
- *
- * utf::string::replace_copy --
- *
- * Results:
- *      Returns a new string with all occurrences of one string replaced by
- *      another.
- *
- * Side effects:
- *      None
- *
- *-----------------------------------------------------------------------------
- */
-
-string
-string::replace_copy(const string& from, // IN
-                     const string& to)   // IN
-   const
-{
-   return string(*this).replace(from, to);
-}
-
-
 /*
  *-----------------------------------------------------------------------------
  *
@@ -1459,7 +1433,7 @@ string::rfind(value_type uc,     // IN
  *
  * utf::string::find_first_of --
  *
- *      Find the first occurrence of 's' in this string.  'i' determines where in
+ *      Find the first occurence of 's' in this string.  'i' determines where in
  *      the current string we start searching for 's'
  *
  * Results:
@@ -1496,7 +1470,7 @@ string::find_first_of(value_type uc,   // IN
  *
  * utf::string::find_first_not_of --
  *
- *      Find the first occurrence of a string NOT in 's' in this string.  'i'
+ *      Find the first occurence of a string NOT in 's' in this string.  'i'
  *      determines where in this string we start searching to NOT 's'.
  *
  * Results:
